'Created by: X
'Copyright: Created by: X
'Web: http://www.createdbyx.com/
'Date: March 27, 2004
'=============================================
Imports Microsoft
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D.Cull
Imports Microsoft.DirectX.Direct3D.FillMode
Imports Microsoft.DirectX.Direct3D.ShadeMode
Imports CBX.Libs


''' <summary>
''' Implements the IGeometry and IMaterialBlended interfaces an uses the PositionColoredTextured vertex structure for storing vertexes.
''' </summary>
Public Class Brush
    Implements Interfaces.IGeometry
    Implements Interfaces.IMaterialBlended
    Implements IEntity, ICloneable, IDisposable


    Protected Friend mobjVerts() As Direct3D.CustomVertex.PositionColoredTextured
    Protected Friend mintPrimType As Direct3D.PrimitiveType = PrimitiveType.TriangleFan
    Protected Friend mintTexture As Integer = -1
    Protected Friend mobjState As New Direct3D.TextureState()
    Public Text As String
    Public Rect As Rectangle
    Public xTile As Single = 1
    Public yTile As Single = 1

    ' Protected Friend mintStageCount As Integer = 1
    Protected Friend mblnSelected As Boolean = False

    ''' <summary>
    ''' Gets the array of the internally stored vertexes.
    ''' </summary>
    ''' <returns>Returns the array of the internally stored vertexes.</returns>
    Public Function GetVerts() As Direct3D.CustomVertex.PositionColoredTextured()
        Return mobjVerts
    End Function

    ''' <summary>
    ''' Gets the number primitives based on the numbers of verts and the PrimitiveType property.
    ''' </summary>
    ''' <returns>Returns the number primitives based on the numbers of verts and the PrimitiveType property.</returns>
    Public Function GetPrimitiveCount() As Integer Implements Interfaces.IGeometry.GetPrimitiveCount
        Select Case mintPrimType
            Case PrimitiveType.PointList
                Return mobjVerts.Length

            Case PrimitiveType.LineStrip, PrimitiveType.LineList
                If mobjVerts.Length < 2 Then Return 0 Else Return mobjVerts.Length - 1

            Case PrimitiveType.TriangleList
                If mobjVerts.Length < 3 Then Return 0 Else Return mobjVerts.Length \ 3

            Case PrimitiveType.TriangleStrip, PrimitiveType.TriangleFan
                If mobjVerts.Length < 3 Then Return 0 Else Return mobjVerts.Length - 2
        End Select
    End Function

    ''' <summary>
    ''' Gets/Sets the number of vertexes that are stored.
    ''' </summary>
    ''' <returns>Returns the number of vertexes that are stored.</returns>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the value specified is less then or equal to 0.
    ''' </exception>
    Public Property VertexCount() As Integer Implements Interfaces.IGeometry.VertexCount
        Get
            Return mobjVerts.GetUpperBound(0)
        End Get
        Set(ByVal Value As Integer)
            If Value <= 0 Then Throw New ArgumentOutOfRangeException("Value", "Value must be 1 or higher.")
            ReDim Preserve mobjVerts(Value - 1)
        End Set
    End Property

    ''' <summary>
    ''' Retrieves the vertex format for the internaly stored vertex structure.
    ''' </summary>
    ''' <returns>Retrieves the vertex format for the internaly stored vertex structure.</returns>
    Public ReadOnly Property Format() As Direct3D.VertexFormats Implements Interfaces.IGeometry.Format
        Get
            Return Direct3D.CustomVertex.PositionColoredTextured.Format
        End Get
    End Property

    ''' <summary>
    ''' Retrieves the size of the internaly stored vertex structure.
    ''' </summary>
    ''' <returns>Retrieves the size of the internaly stored vertex structure.</returns>
    Public ReadOnly Property StrideSize() As Integer Implements Interfaces.IGeometry.StrideSize
        Get
            Return Direct3D.CustomVertex.PositionColoredTextured.StrideSize
        End Get
    End Property

    ''' <summary>
    ''' Gets/Sets the manner in which the vertexes are stored.
    ''' </summary>
    ''' <returns>Returns the manner in which the vertexes are stored.</returns>
    Public Property PrimitiveType() As Direct3D.PrimitiveType Implements Interfaces.IGeometry.PrimitiveType
        Get
            Return mintPrimType
        End Get
        Set(ByVal Value As Direct3D.PrimitiveType)
            mintPrimType = Value
        End Set
    End Property

    ''' Gets/Sets the index of a texture blending TextureState type.
    ''' </summary>
    ''' <returns>Returns the index of a texture blending TextureState type.</returns>
    Public Function GetState() As Microsoft.DirectX.Direct3D.TextureState Implements Interfaces.IMaterialBlended.GetState
        Return mobjState
    End Function

    ''' <summary>
    ''' Gets/Sets the index of a texture at a specified stage.
    ''' </summary>
    ''' <param name="Stage">Specifies the stage of the texture.</param>
    ''' <returns>Returns the index of a texture at a specified stage.</returns>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Stage is less then 0 or greater than the number of stages minus 1.
    ''' </exception>
    Public Property Texture() As Integer Implements Interfaces.IMaterialBlended.Texture
        Get
            Return mintTexture
        End Get
        Set(ByVal Value As Integer)
            mintTexture = Value
        End Set
    End Property

#Region " SetVertex overloads ... "
    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="V">A reference to a PositionColoredTextured vertex type.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal V As Direct3D.CustomVertex.PositionColoredTextured)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index) = V
    End Sub

    ''' <summary>
    ''' Sets all vertexes to the same color.
    ''' </summary>
    ''' <param name="Color">A color to set all the vertexes to.</param>
    Public Sub SetVertex(ByVal Color As Integer)
        Dim idx As Integer

        For idx = 0 To mobjVerts.Length - 1
            mobjVerts(idx).Color = Color
        Next
    End Sub

    ''' <summary>
    ''' Sets all vertexes to the same color.
    ''' </summary>
    ''' <param name="Color">A color to set all the vertexes to.</param>
    Public Sub SetVertex(ByVal Color As Color)
        SetVertex(Color.ToArgb)
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexs to set.</param>
    ''' <param name="V">A reference to an array of PositionColoredTextured vertex types.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal V() As Direct3D.CustomVertex.PositionColoredTextured)
        Dim idx As Integer

        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        If Index + V.Length > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index + V.Length")

        ' copy verts
        Array.Copy(V, V.GetLowerBound(0), mobjVerts, Index, V.Length)
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="Position">The position to set the vertex at.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Position As Vector3)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).SetPosition(Position)
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="X">The X position to set the vertex at.</param>
    ''' <param name="Y">The Y position to set the vertex at.</param>
    ''' <param name="Z">The Z position to set the vertex at.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal X As Single, ByVal Y As Single, ByVal Z As Single)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).SetPosition(New Vector3(X, Y, Z))
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="Position">The position to set the vertex at.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Position As Vector3, ByVal Color As Color)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).SetPosition(Position)
        mobjVerts(Index).Color = Color.ToArgb
    End Sub

    ''' <summary>
    ''' Sets the vertex data starting at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexes to set.</param>
    ''' <param name="Position">The array of positions to set the vertexes at.</param>
    ''' <param name="Color">Specifies the Color to set the vertexes as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    '''     Or if Index + Position.Length is greater then the number of vertexes.
    ''' </exception>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a ArgumentNullException Exception if Position is nothing.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Position() As Vector3, ByVal Color As Integer)
        Dim idx As Integer

        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        If Position Is Nothing Then Throw New ArgumentNullException("Position")
        If Index + Position.Length - 1 > mobjVerts.Length Then Throw New ArgumentOutOfRangeException("Index + Position.Length")

        For idx = 0 To Position.Length - 1
            mobjVerts(Index + idx).SetPosition(Position(idx))
            mobjVerts(Index + idx).Color = Color
        Next
    End Sub

    ''' <summary>
    ''' Sets the vertex data starting at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexes to set.</param>
    ''' <param name="Position">The array of positions to set the vertexes at.</param>
    ''' <param name="Color">Specifies the Color to set the vertexes as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    '''     Or if Index + Position.Length is greater then the number of vertexes minus 1.
    ''' </exception>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a ArgumentNullException Exception if Position is nothing.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Position() As Vector3, ByVal Color As Color)
        SetVertex(Index, Position, Color.ToArgb)
    End Sub

    ''' <summary>
    ''' Sets the vertex data starting at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexes to set.</param>
    ''' <param name="Position">The array of positions to set the vertexes at.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    '''     Or if Index + Position.Length is greater then the number of vertexes.
    ''' </exception>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a ArgumentNullException Exception if Position is nothing.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Position() As Vector3)
        Dim idx As Integer

        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        If Position Is Nothing Then Throw New ArgumentNullException("Position")
        If Index + Position.Length - 1 > mobjVerts.Length Then Throw New ArgumentOutOfRangeException("Index + Position.Length")

        For idx = 0 To Position.Length - 1
            mobjVerts(Index + idx).SetPosition(Position(idx))
        Next
    End Sub

    ''' <summary>
    ''' Sets the vertex data starting at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexes to set.</param>
    ''' <param name="UV">The array of positions to set the vertexes texture coordanants at.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    '''     Or if Index + UV.Length is greater then the number of vertexes minus 1.
    ''' </exception>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a ArgumentNullException Exception if UV is nothing.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal UV() As Vector2)
        Dim idx As Integer

        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        If UV Is Nothing Then Throw New ArgumentNullException("UV")
        If Index + UV.Length - 1 > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index + UV.Length")

        For idx = 0 To UV.Length - 1
            mobjVerts(Index + idx).Tu = UV(idx).X
            mobjVerts(Index + idx).Tv = UV(idx).Y
        Next
    End Sub

    ''' <summary>
    ''' Sets the vertex data starting at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexes to set.</param>
    ''' <param name="UV">The array of positions to set the vertexes texture coordanants at.</param>
    ''' <param name="Color">Specifies the Color to set the vertexes as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    '''     Or if Index + UV.Length is greater then the number of vertexes.
    ''' </exception>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a ArgumentNullException Exception if UV is nothing.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal UV() As Vector2, ByVal Color As Integer)
        Dim idx As Integer

        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        If UV Is Nothing Then Throw New ArgumentNullException("UV")
        If Index + UV.Length - 1 > mobjVerts.Length Then Throw New ArgumentOutOfRangeException("Index + UV.Length")

        For idx = 0 To UV.Length - 1
            mobjVerts(Index + idx).Tu = UV(idx).X
            mobjVerts(Index + idx).Tv = UV(idx).Y
            mobjVerts(Index + idx).Color = Color
        Next
    End Sub

    ''' <summary>
    ''' Sets the vertex data starting at the specified index.
    ''' </summary>
    ''' <param name="Index">The start index of the vertexes to set.</param>
    ''' <param name="UV">The array of positions to set the vertexes texture coordanants at.</param>
    ''' <param name="Color">Specifies the Color to set the vertexes as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    '''     Or if Index + UV.Length is greater then the number of vertexes.
    ''' </exception>
    ''' <exception cref="ArgumentNullException">
    '''     Will throw a ArgumentNullException Exception if UV is nothing.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal UV() As Vector2, ByVal Color As Color)
        SetVertex(Index, UV, Color.ToArgb)
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="X">The X position to set the vertex at.</param>
    ''' <param name="Y">The Y position to set the vertex at.</param>
    ''' <param name="Z">The Z position to set the vertex at.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal X As Single, ByVal Y As Single, ByVal Z As Single, ByVal Color As Color)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).SetPosition(New Vector3(X, Y, Z))
        mobjVerts(Index).Color = Color.ToArgb
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="Position">The position to set the vertex at.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Position As Vector3, ByVal Color As Integer)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).SetPosition(Position)
        mobjVerts(Index).Color = Color
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="X">The X position to set the vertex at.</param>
    ''' <param name="Y">The Y position to set the vertex at.</param>
    ''' <param name="Z">The Z position to set the vertex at.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal X As Single, ByVal Y As Single, ByVal Z As Single, ByVal Color As Integer)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).SetPosition(New Vector3(X, Y, Z))
        mobjVerts(Index).Color = Color
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Color As Color)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Color = Color.ToArgb
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal Color As Integer)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Color = Color
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="UV">Sets the texture UV for the vertex.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal UV As Vector2)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Tu = UV.X
        mobjVerts(Index).Tv = UV.Y
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="tU">Sets the texture U for the vertex.</param>
    ''' <param name="tV">Sets the texture V for the vertex.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal tU As Single, ByVal tV As Single)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Tu = tU
        mobjVerts(Index).Tv = tV
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="UV">Sets the texture UV for the vertex.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal UV As Vector2, ByVal Color As Color)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Tu = UV.X
        mobjVerts(Index).Tv = UV.Y
        mobjVerts(Index).Color = Color.ToArgb
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="tU">Sets the texture U for the vertex.</param>
    ''' <param name="tV">Sets the texture V for the vertex.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal tU As Single, ByVal tV As Single, ByVal Color As Color)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Tu = tU
        mobjVerts(Index).Tv = tV
        mobjVerts(Index).Color = Color.ToArgb
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="UV">Sets the texture UV for the vertex.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal UV As Vector2, ByVal Color As Integer)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Tu = UV.X
        mobjVerts(Index).Tv = UV.Y
        mobjVerts(Index).Color = Color
    End Sub

    ''' <summary>
    ''' Sets the vertex data at the specified index.
    ''' </summary>
    ''' <param name="Index">The index of the vertex to set.</param>
    ''' <param name="tU">Sets the texture U for the vertex.</param>
    ''' <param name="tV">Sets the texture V for the vertex.</param>
    ''' <param name="Color">Specifies the Color to set the vertex as.</param>
    ''' <exception cref="ArgumentOutOfRangeException">
    '''     Will throw a ArgumentOutOfRangeException Exception if the Index is less then 0 or greater than the number of vertexes minus 1.
    ''' </exception>
    Public Sub SetVertex(ByVal Index As Integer, ByVal tU As Single, ByVal tV As Single, ByVal Color As Integer)
        If Index < 0 Then Throw New ArgumentOutOfRangeException("Index")
        If Index > mobjVerts.Length - 1 Then Throw New ArgumentOutOfRangeException("Index")
        mobjVerts(Index).Tu = tU
        mobjVerts(Index).Tv = tV
        mobjVerts(Index).Color = Color
    End Sub
#End Region

    Public Function Kind() As IEntity.EntityType Implements IEntity.Kind
        Return IEntity.EntityType.Brush
    End Function

    Public Property Selected() As Boolean Implements IEntity.Selected
        Get
            Return mblnSelected
        End Get
        Set(ByVal Value As Boolean)
            mblnSelected = Value
        End Set
    End Property

    Public Function Clone() As Object Implements System.ICloneable.Clone
        Dim B As New Brush()
        Dim idx As Integer

        B.GetState.FromTextureState(Me.GetState)
        B.Texture = Me.Texture
        B.PrimitiveType = Me.PrimitiveType
        B.Selected = Me.Selected
        B.SetVertex(0, Me.GetVerts)
    End Function

    Friend Sub Dispose() Implements System.IDisposable.Dispose
        Erase mobjVerts
        mobjVerts = Nothing
    End Sub
End Class