'Created by: X
'Copyright: Created by: X
'Web: http://www.createdbyx.com/
'Date: April 7, 2004
'=============================================
Imports Microsoft
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D
Imports Microsoft.DirectX.Direct3D.Cull
Imports Microsoft.DirectX.Direct3D.FillMode
Imports Microsoft.DirectX.Direct3D.ShadeMode
Imports Microsoft.DirectX.Direct3D.TransformType
Imports Microsoft.DirectX.Direct3D.PrimitiveType
Imports CBX.Libs

''' <summary>
''' Contais a collection of texture types.
''' </summary>
Public Class TextureCollection
    Inherits CollectionBase
    Implements IDisposable


    ''' <summary>
    ''' Swaps the location of two objects in the collection.
    ''' </summary>
    ''' <param name="Item1">The first reference to a texture type previously added to the collection.</param>
    ''' <param name="Item2">The second reference to a texture type previously added to the collection.</param>
    Public Sub Swap(ByVal Item1 As Texture, ByVal Item2 As Texture)
        Dim Index1, Index2 As Integer

        Index1 = Me.IndexOf(Item1)
        Index2 = Me.IndexOf(Item2)

        ' do err chech
        If Index1 < 0 Or Index1 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Item1", "The item was not previosly added to this collection.")
        If Index2 < 0 Or Index2 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Item2", "The item was not previosly added to this collection.")
        ' dont need to do anything
        If Index1 = Index2 Then Exit Sub

        Swap(Index1, Index2)
    End Sub

    ''' <summary>
    ''' Swaps the location of two items in the collection.
    ''' </summary>
    ''' <param name="Index1">The index of the first item.</param>
    ''' <param name="Index2">The index of the second item.</param>
    Public Sub Swap(ByVal Index1 As Integer, ByVal Index2 As Integer)
        Dim intFirst, intLast As Integer
        Dim objFirst, objLast As Texture

        ' do err chech
        If Index1 < 0 Or Index1 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Index1")
        If Index2 < 0 Or Index2 > Me.Count - 1 Then Throw New ArgumentOutOfRangeException("Index2")
        ' dont need to do anything
        If Index1 = Index2 Then Exit Sub

        intFirst = CType(IIf(Index1 < Index2, Index1, Index2), Integer)
        intLast = CType(IIf(Index1 > Index2, Index1, Index2), Integer)

        ' store items
        objFirst = Me(intFirst)
        objLast = Me(intLast)

        ' remove from list
        Me.Remove(intLast)
        Me.Remove(intFirst)
        ' reinsert
        Me.Insert(intFirst, objFirst)
        Me.Insert(intLast, objLast)
    End Sub

    ''' <summary>
    ''' Disposes of all items in the collection by calling the Clear method.
    ''' </summary>
    Public Sub Dispose() Implements System.IDisposable.Dispose
        Me.Clear()
    End Sub

    ''' <summary>
    ''' Adds a texture type to the collection.
    ''' </summary>
    ''' <param name="Item">A reference to a texture type.</param>
    ''' <returns>Returns the index that the item was added to.</returns>
    Public Function Add(ByVal Item As Texture) As Integer
        Return Me.InnerList.Add(Item)
    End Function

    ''' <summary>
    ''' Returns the index of a texture type previously added to the collection.
    ''' </summary>
    ''' <param name="Item">A reference to a texture type that was previously added.</param>
    Public Function IndexOf(ByVal Item As Texture) As Integer
        Return Me.InnerList.IndexOf(Item)
    End Function

    ''' <summary>
    ''' Inserts a texture type into the collection.
    ''' </summary>
    ''' <param name="Index">The index of the item to be retrieved.</param>
    ''' <param name="Item">A reference to a texture type.</param>
    Public Sub Insert(ByVal Index As Integer, ByVal Item As Texture)
        Me.InnerList.Insert(Index, Item)
    End Sub

    ''' <summary>
    ''' Gets/Sets a item in the collection.
    ''' </summary>
    ''' <param name="Index">The index of the item to be retrieved.</param>
    ''' <returns>Returns a reference to a texture type.</returns>
    Default Public Property Item(ByVal Index As Integer) As Texture
        Get
            Return CType(Me.InnerList(Index), Texture)
        End Get
        Set(ByVal Value As Texture)
            Me.InnerList(Index) = Value
        End Set
    End Property

    ''' <summary>
    ''' Removes an item.
    ''' </summary>
    ''' <param name="Item">A reference to a texture type previously added to the collection.</param>
    Public Sub Remove(ByVal Item As Texture)
        Me.InnerList.Remove(Item)
    End Sub

    ''' <summary>
    ''' Removes an item.
    ''' </summary>
    ''' <param name="Index">The index of the item to be removed.</param>
    Public Sub Remove(ByVal Index As Integer)
        Me.InnerList.RemoveAt(Index)
    End Sub

    ''' <summary>
    ''' Removes a number of items.
    ''' </summary>
    ''' <param name="Index">The index of the first item to be removed.</param>
    ''' <param name="Count">The number of items to removed.</param>
    Public Sub Remove(ByVal Index As Integer, ByVal Count As Integer)
        Me.InnerList.RemoveRange(Index, Count)
    End Sub

    ''' <summary>
    ''' Removes a number of items from the collection.
    ''' </summary>
    ''' <param name="Indexes">The index(s) of the items to be removed.</param>
    Public Sub Remove(ByVal Indexes() As Integer)
        Dim idx As Integer = Indexes.GetUpperBound(0)

        While idx > Indexes.GetLowerBound(0)
            Me.Remove(Indexes(idx))
            idx -= 1
        End While
    End Sub
End Class
